<?php

namespace Drupal\layout_settings\Plugin\LayoutSettings;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Ajax\AjaxHelperTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Markup;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file\Element\ManagedFile;
use Drupal\file\Entity\File;
use Drupal\layout_settings\Plugin\LayoutSettingsBase;
use Drupal\layout_settings\Plugin\LayoutSettingsManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * @LayoutSettings(
 *  id = "background",
 *  label = @Translation("Background"),
 *  context = {
 *    "layout",
 *    "rows",
 *    "regions",
 *  }
 * )
 */
class Background extends LayoutSettingsBase {

  use BackgroundCssTrait;
  use AjaxHelperTrait;
  use StringTranslationTrait;

  protected $destination = 'public://layout_settings/background';

  /**
   * The image style entity storage.
   *
   * @var \Drupal\image\ImageStyleStorageInterface
   */
  protected $imageStyleStorage;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LayoutSettingsManager $layout_settings_manager, EntityStorageInterface $image_style_storage, Request $request, RendererInterface $renderer) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $layout_settings_manager);
    $this->imageStyleStorage = $image_style_storage;
    $this->request = $request;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.layout_settings'),
      $container->get('entity_type.manager')->getStorage('image_style'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('renderer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build(&$build, $context) {
    /** @var \Drupal\Component\Uuid\Php $uuid */
    $uuid_service = \Drupal::service('uuid');
    $uuid = 'layout-settings-';
    $uuid .= $uuid_service->generate();
    switch ($context[0]) {
      case 'layout':
        $this->generateBackgroundStyle('#' . $uuid, $build);
        $region = $context[1];
        if ($region === 'wrapper') {
          $build['#parent_attributes']['id'] = $uuid;
        }
        else {
          $build['#attributes']['id'] = $uuid;
        }
        break;

      case 'rows':
        break;

      case 'regions':
        $this->generateBackgroundStyle('#' . $uuid, $build);
        $region = $context[1];
        $build[$region]['#attributes']['id'] = $uuid;
        break;

      default:
        break;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultValues() {
    return self::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array &$form, FormStateInterface $form_state) {
    $element = [];
    $data = $this->configuration;

    $image = !empty($data['image']) ? $data['image'] : [''];
    $element['image'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Image'),
      '#description' => $this->t('Allowed extensions: gif png jpg jpeg'),
      '#upload_validators' => [
        'file_validate_extensions' => ['gif png jpg jpeg'],
        // Pass the maximum file size in bytes.
        'file_validate_size' => [1 * 1024 * 1024],
      ],
      '#theme' => 'image_widget',
      '#upload_location' => $this->destination,
      '#default_value' => $image,
      '#field_name' => 'render_property_background_image',
      '#process' => [
        [get_class($this), 'processManagedFile'],
      ],
    ];

    $element += $this->settingsForm($data);

    $element['css_settings']['image_selector']['#access'] = FALSE;
    $element['css_settings']['token_help']['#access'] = FALSE;


    return $element;
  }

  /**
   * Render API callback: Expands the managed_file element type.
   *
   * Expands the file type to include Upload and Remove buttons, as well as
   * support for a default value.
   */
  public static function processManagedFile(&$element, FormStateInterface $form_state, &$complete_form) {
    $element = ManagedFile::processManagedFile($element, $form_state, $complete_form);
    $element['remove_button']['#submit'][0] = [
      self::class,
      'fileManagedFileSubmit2',
    ];

    $triggering = $form_state->getTriggeringElement();
    if (empty($triggering)) {
      return $element;
    }
    array_pop($triggering['#parents']);
    $values = $form_state->getValues();
    foreach ($triggering['#parents'] as $item) {
      $values = $values[$item];
    }
    if (!empty($values['fids'])) {
      $images = $values['fids'];
    }
    else {
      $images = [];
    }
    //$images = $form_state->getValue('background')['image']['fids'];
    if (!empty($images)) {
      foreach ($images as $image) {
        /* @var \Drupal\file\FileInterface $file */
        if ($file = File::load($image)) {
          $file->setPermanent();
          $file->save();

          $file_usage = \Drupal::service('file.usage');
          $file_usage->add($file, 'layout_settings', 'user', 0);
        }
      }
    }

    return $element;
  }

  public static function fileManagedFileSubmit2($form, FormStateInterface $form_state) {
    // Determine whether it was the upload or the remove button that was clicked,
    // and set $element to the managed_file element that contains that button.
    $parents = $form_state->getTriggeringElement()['#array_parents'];
    $button_key = array_pop($parents);
    $element = NestedArray::getValue($form, $parents);

    // No action is needed here for the upload button, because all file uploads on
    // the form are processed by \Drupal\file\Element\ManagedFile::valueCallback()
    // regardless of which button was clicked. Action is needed here for the
    // remove button, because we only remove a file in response to its remove
    // button being clicked.
    if ($button_key == 'remove_button') {
      $fids = array_keys($element['#files']);
      // Get files that will be removed.
      if ($element['#multiple']) {
        $remove_fids = [];
        foreach (Element::children($element) as $name) {
          if (strpos($name, 'file_') === 0 && $element[$name]['selected']['#value']) {
            $remove_fids[] = (int) substr($name, 5);
          }
        }
        $fids = array_diff($fids, $remove_fids);
      }
      else {
        // If we deal with single upload element remove the file and set
        // element's value to empty array (file could not be removed from
        // element if we don't do that).
        $remove_fids = $fids;
        $fids = [];
      }
      /*
      foreach ($remove_fids as $fid) {
        // If it's a temporary file we can safely remove it immediately, otherwise
        // it's up to the implementing module to remove usages of files to have them
        // removed.
        if ($element['#files'][$fid] && $element['#files'][$fid]->isTemporary()) {
          $element['#files'][$fid]->delete();
        }
      }
      */
      // Update both $form_state->getValues() and FormState::$input to reflect
      // that the file has been removed, so that the form is rebuilt correctly.
      // $form_state->getValues() must be updated in case additional submit
      // handlers run, and for form building functions that run during the
      // rebuild, such as when the managed_file element is part of a field widget.
      // FormState::$input must be updated so that
      // \Drupal\file\Element\ManagedFile::valueCallback() has correct information
      // during the rebuild.
      $form_state->setValueForElement($element['fids'], implode(' ', $fids));
      NestedArray::setValue($form_state->getUserInput(), $element['fids']['#parents'], implode(' ', $fids));
    }

    // Set the form to rebuild so that $form is correctly updated in response to
    // processing the file removal. Since this function did not change $form_state
    // if the upload button was clicked, a rebuild isn't necessary in that
    // situation and calling $form_state->disableRedirect() would suffice.
    // However, we choose to always rebuild, to keep the form processing workflow
    // consistent between the two buttons.
    $form_state->setRebuild();
  }

  public function generateBackgroundStyle($selector, &$build) {
    $settings = $this->configuration;
    $css_settings = $settings['css_settings'];
    // Replace views tokens.
    $css_settings['image_selector'] = $selector;
    $image_style = $settings['image_style'] ? $settings['image_style'] : NULL;
    $path_format = $css_settings['image_path_format'];
    $selectors = explode(PHP_EOL, trim($css_settings['image_selector']));

    // Filter out empty selectors.
    $selectors = array_map(function ($value) {
      return trim($value, ',');
    }, $selectors);

    if (!empty($settings['image'][0])) {
      $fid = $settings['image'][0];
      $file = File::load($fid);
    }
    else {
      $file = NULL;
    }

    if ($file !== NULL && $image_style) {
      $style = $this->imageStyleStorage->load($image_style);
      $file_url = $style->buildUrl($file->getFileUri());
    }
    elseif ($file !== NULL) {
      $file_url = file_create_url($file->getFileUri());
    }
    else {
      $file_url = NULL;
    }

    if ($file_url !== NULL) {
      $image_url = $path_format === 'absolute' ? $file_url : file_url_transform_relative($file_url);
    }
    else {
      $image_url = NULL;
    }

    $css = $this->getBackgroundImageCss($image_url, $css_settings);

    // Define unique key to prevent collisions when displaying multiple
    // background images on the same page.
    $html_head_key = 'image_formatter_css__s' . sha1($selector);

    $style_element = [
      '#type' => 'html_tag',
      '#tag' => 'style',
      '#attributes' => [
        'media' => $css['media'],
      ],
      '#value' => Markup::create($css['style']),
    ];

    if ($this->isAjax() || $this->request->isXmlHttpRequest()) {
      $variables['#attached']['drupalSettings']['image_formatter_css'][$html_head_key] =
        $this->renderer->renderPlain($style_element);
    }
    else {
      $build['#attached']['html_head'][] = [$style_element, $html_head_key];
    }
  }

}
