<?php

namespace Drupal\layout_settings\Plugin;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Layout\LayoutDefinition;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Render\Element;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\layout_settings\Annotation\LayoutSettings;
use Traversable;

/**
 * Provides the Layout settings plugin manager.
 */
class LayoutSettingsManager extends DefaultPluginManager {

  use StringTranslationTrait;

  /**
   * Constructs a new LayoutSettingsManager object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/LayoutSettings', $namespaces, $module_handler, LayoutSettingsInterface::class, LayoutSettings::class);

    $this->alterInfo('layout_settings_info');
    $this->setCacheBackend($cache_backend, 'layout_settings_plugins');
  }

  /**
   * {@inheritdoc}
   */
  public function processDefinition(&$definition, $plugin_id) {
    parent::processDefinition($definition, $plugin_id);

    // Add the module or theme path to the 'path'.
    if ($this->moduleHandler->moduleExists($definition['provider'])) {
      $definition['provider_type'] = 'module';
      $definition['definition_path'] = $this->moduleHandler->getModule($definition['provider'])
        ->getPath();
    }
    else {
      $definition['definition_path'] = '';
    }
  }

  /**
   *
   */
  public function getDefaultValues(array &$defaults = []) {
    $contexts = static::getSettingContexts();
    $definitions = $this->getDefinitions();
    $defaults['icon_map'] = [];
    foreach ($definitions as $id => $def) {
      /** @var \Drupal\layout_settings\Plugin\LayoutSettingsInterface $setting_instance */
      $setting_instance = $this->createInstance($id);
      foreach ($def['context'] as $context) {
        foreach ($contexts as $item) {
          if (!empty($defaults['icon_map'][$context][$item])) {
            $defaults['icon_map_settings'][$context][$item][$id] = $setting_instance->defaultValues();
          }
        }
      }
    }
  }

  public static function getSettingContexts() {
    return ['layout', 'rows', 'regions'];
  }

  /**
   *
   */
  public function buildSettingsForm(array $form, FormStateInterface $form_state, array $configuration, LayoutDefinition $definition) {
    $icon_map = $definition->getIconMap();
    if (empty($icon_map)) {
      return $form;
    }
    $form['layout_settings'] = [
      '#type' => 'vertical_tabs',
      '#tree' => TRUE,
    ];

    $form['icon_map'] = [
      '#type' => 'details',
      '#title' => $this->t('Select context settings to modify'),
    ];

    $contexts = static::getSettingContexts();
    foreach ($contexts as $context) {
      $form['icon_map'][$context] = [
        '#type' => 'details',
        '#title' => $this->t($context),
      ];
    }
    $form['icon_map']['layout']['wrapper'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Wrapper', [], ['context' => 'layout_settings']),
      '#default_value' => !empty($configuration['icon_map']['layout']['wrapper']) ? $configuration['icon_map']['layout']['wrapper'] : NULL,
    ];
    $form['icon_map']['layout']['layout'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Layout', [], ['context' => 'layout_settings']),
      '#default_value' => !empty($configuration['icon_map']['layout']['layout']) ? $configuration['icon_map']['layout']['layout'] : NULL,
    ];

    foreach ($icon_map as $row_id => $row) {
      $form['icon_map']['rows'][$row_id] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Row %id', ['%id' => $row_id]),
        '#default_value' => !empty($configuration['icon_map']['rows'][$row_id]) ? $configuration['icon_map']['rows'][$row_id] : NULL,
      ];
      foreach ($row as $column_id => $column) {
        $form['icon_map']['regions'][$column] = [
          '#type' => 'checkbox',
          '#title' => $this->t('Region %id', ['%id' => $column]),
          '#default_value' => !empty($configuration['icon_map']['regions'][$column]) ? $configuration['icon_map']['regions'][$column] : NULL,
        ];
      }
    }

    $definitions = $this->getDefinitions();
    foreach ($definitions as $id => $def) {
      /** @var \Drupal\layout_settings\Plugin\LayoutSettingsInterface $setting_instance */
      foreach ($def['context'] as $context) {
        foreach (Element::children($form['icon_map'][$context]) as $item) {
          $st = [];
          $conf = !empty($configuration['icon_map_settings'][$context][$item][$id]) ? $configuration['icon_map_settings'][$context][$item][$id] : [];
          $setting_instance = $this->createInstance($id, $conf);
          $setting_form = $setting_instance->buildForm($st, $form_state);
          $selector = 'layout_settings[icon_map][' . $context . '][' . $item . ']';
          if (empty($form['icon_map_settings'][$context][$item])) {
            $form['icon_map_settings'][$context][$item] = [
              '#type' => 'details',
              '#title' => $this->t('Context %id settings', ['%id' => $item]),
              '#states' => [
                'visible' => [
                  '[name="' . $selector . '"]' => ['checked' => TRUE],
                ],
              ],
            ];
          }
          $form['icon_map_settings'][$context][$item][$id . '_status'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Enable :plugin settings', [':plugin' => $def['label']]),
            '#default_value' => !empty($configuration['icon_map_settings'][$context][$item][$id . '_status']) ? $configuration['icon_map_settings'][$context][$item][$id . '_status'] : NULL,
          ];
          $selector = 'layout_settings[icon_map_settings][' . $context . '][' . $item . '][' . $id . '_status]';
          $form['icon_map_settings'][$context][$item][$id] = [
              '#type' => 'details',
              '#title' => $def['label'],
              '#open' => TRUE,
              '#states' => [
                'visible' => [
                  '[name="' . $selector . '"]' => ['checked' => TRUE],
                ],
              ],
            ] + $setting_form;
        }
      }
    }

    if (count(Element::children($form['icon_map']['rows'])) < 2) {
      $form['icon_map']['rows']['#access'] = FALSE;
      $form['icon_map_settings']['rows']['#access'] = FALSE;
    }

    return $form;
  }

  public function submitSettingsForm(array &$form, FormStateInterface $form_state, array &$configuration, LayoutDefinition $definition) {
    $definitions = $this->getDefinitions();
    $values = $form_state->getValues();
    $configuration['icon_map'] = $values['icon_map'];
    $configuration['icon_map_settings'] = $values['icon_map_settings'];
    foreach ($definitions as $id => $def) {
      /** @var \Drupal\layout_settings\Plugin\LayoutSettingsInterface $setting_instance */
      $setting_instance = $this->createInstance($id, $configuration);
      $setting_instance->submitForm($form, $form_state);
      $configuration = $setting_instance->get('configuration');
    }
  }

  public function validateSettingsForm(array &$form, FormStateInterface $form_state, array &$configuration, LayoutDefinition $definition) {
    $definitions = $this->getDefinitions();
    foreach ($definitions as $id => $def) {
      /** @var \Drupal\layout_settings\Plugin\LayoutSettingsInterface $setting_instance */
      $setting_instance = $this->createInstance($id);
      $setting_instance->submitForm($form, $form_state);
    }
  }

  public function getIconHighlighted($width = 125, $height = 150, $stroke_width = NULL, $padding = NULL, LayoutDefinition $definition, string $highlight) {
    $icon = [];
    if ($icon_path = $definition->getIconPath()) {
      $icon = [
        '#theme' => 'image',
        '#uri' => $icon_path,
        '#width' => $width,
        '#height' => $height,
        '#alt' => $definition->getLabel(),
      ];
    }
    elseif ($icon_map = $definition->getIconMap()) {
      $icon_builder = \Drupal::service('layout.icon_builder');
      $icon_builder->setId($definition->id())
        ->setLabel($definition->getLabel())
        ->setWidth($width)
        ->setHeight($height);
      if ($padding) {
        $icon_builder->setPadding($padding);
      }
      if ($stroke_width) {
        $icon_builder->setStrokeWidth($stroke_width);
      }
      $icon = $icon_builder->buildAndHighlight($icon_map, $highlight);
    }
    return $icon;
  }

  public function build(array $regions, $configuration, &$build) {
    $contexts = self::getSettingContexts();
    $definitions = $this->getDefinitions();
    foreach ($contexts as $context) {
      foreach ($configuration['icon_map_settings'][$context] as $region => $settings) {
        if (empty($configuration['icon_map'][$context][$region])) {
          continue;
        }
        foreach ($settings as $plugin_id => $setting) {
          if (!empty($definitions[$plugin_id]) && !empty($settings[$plugin_id . '_status'])) {
            /** @var \Drupal\layout_settings\Plugin\LayoutSettingsInterface $setting_instance */
            $setting_instance = $this->createInstance($plugin_id, $setting);
            $setting_instance->build($build, [
              $context,
              $region,
            ]);
          }
        }
      }
    }
  }

}
